/*
 *      SpxClient is a silly little program that tests the SPX layer.
 *      Writen by Jay.Schulist@spacs.k12.wi.us
 */

/* Standard libs */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <signal.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>

/* glibc specials */
#include <sys/types.h>
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netipx/ipx.h>

#include "Spx.h"        /* Our goodies ;) */

/* Functions Protos */
char *in_ntoa(unsigned long in);
unsigned long in_aton(const char *str);
int HexDump(unsigned char *pkt_data, int pkt_len);

const char ProgName[] = "SpxClient";
const char Title[] = "Written by Jay.Schulist@spacs.k12.wi.us";

void main(void)
{
	int SockSPX;
	unsigned char Node[] = "000001";	/* Node to connect to */
	int opt, err, i, len, count;

	struct sockaddr_ipx spx, dest;
        int     result;
	char    msg[100];

	printf("%s: %s\n", ProgName, Title);

	/* Sync to be safe */
	sync();

	/* Open AF_SPX, SEQPACKET socket */
        SockSPX = socket(AF_SPX, SOCK_SEQPACKET, 0);
        if(SockSPX < 0)
        {
                perror("SpxClient: socket");
                exit(1);
        }
        else
                printf("%s: socket(%d) complete\n", ProgName, SockSPX);

	/* Set socket to reuse address */
        opt = 1;
        err = setsockopt(SockSPX, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
        if(err < 0)
        {
                perror("SpxClient: setsockopt");
                exit(1);
        }
        else
                printf("%s: setsockopt(%d) complete\n", ProgName, err);

	/* Setup port to connect to */	
	memset(&spx, 0, sizeof (spx));
	spx.sipx_family 	= AF_SPX;		/* Proto family */
	spx.sipx_network	= htonl(0xC0A80113);	/* Set Con. network */
	spx.sipx_port 		= 0;			/* Our Port */
	spx.sipx_type 		= 5;			/* Packet type */
	for(i=0; i<IPX_NODE_LEN; i++)			/* Set Con. Node */
		spx.sipx_node[i] = Node[i];

	/* Bind our port to the interface(s) for serving */
        err = bind(SockSPX, (struct sockaddr *)&spx, sizeof(spx));
        if(err < 0)
        {
                perror("SpxClient: bind");
                exit(1);
        }
        else
                printf("%s: bind(%d) complete\n", ProgName, err);

        len = sizeof(spx);
        err = getsockname(SockSPX, (struct sockaddr *)&spx, &len);
        if(err < 0)
        {
                perror("SpxClient: getsockname");
                exit(1);
        }
        else
                printf("%s: getsockname(%d) complete\n", ProgName, err);

	/* Connect to remote host on PORT */
	dest = spx;			/* This is *Very* silly */
        dest.sipx_port = htons(PORT);
	err = connect(SockSPX, (struct sockaddr *)&dest, sizeof(dest));
	if(err < 0)
	{
		perror("SpxClient: connect");
		exit(1);
	}
	else
	{
		printf("%s: connect(%d) complete", ProgName, err);
		printf(" to %08X:%02X%02X%02X%02X%02X%02X:%04X\n",
                        htonl(dest.sipx_network),
                        dest.sipx_node[0], dest.sipx_node[1],
                        dest.sipx_node[2], dest.sipx_node[3],
                        dest.sipx_node[4], dest.sipx_node[5],
                        htons(dest.sipx_port));
	}

	count = 1;
	while (1)
	{
		memset(msg, 0, sizeof(msg));
		sprintf(msg, "Message %d from SpxClient!", count);

		result = write(SockSPX, msg, sizeof(msg));
		if(result < 0)
                {
                        perror("SpxClient: socket write1");
                        exit (result);
                }
                else
                        printf("SpxClient: write1 %d bytes\n", result);

		count++;

		memset(msg, 0, sizeof(msg));
		result = read(SockSPX, msg, sizeof(msg));
                if(result < 0)
                {
                        perror("SpxClient: socket read1: ");
                        exit (result);
                }
                else
                {
                        printf("%s: read %d bytes\n", ProgName, result);
                        printf("%s: %s\n", ProgName, msg);
                }

//		for(i=0; i<100000; i++);

		printf("Press any key to continue!\n");
		getchar();
	}
	return;
}
