/*
 *	SpxServer is a silly little program that tests the SPX layer.
 *	Writen by Jay.Schulist@spacs.k12.wi.us
 */

/* Standard libs */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <signal.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>

/* glibc specials */
#include <sys/types.h>
#include <sys/param.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netipx/ipx.h>

#include "Spx.h"	/* Our goodies ;) */

/* Functions Protos */
char *in_ntoa(unsigned long in);
unsigned long in_aton(const char *str);
int HexDump(unsigned char *pkt_data, int pkt_len);

const char ProgName[] = "SpxServer";
const char Title[] = "Written by Jay.Schulist@spacs.k12.wi.us";

void main(void)
{
	int SockSPX, NewSock;
	struct sockaddr_ipx spx;
        int     result;
	char    msg[100];
	int opt, err, len, count;

	printf("%s: %s\n", ProgName, Title);

	/* Sync just to be safe */
	sync();

	/* Open AF_SPX, SEQPACKET socket */
	SockSPX = socket(AF_SPX, SOCK_SEQPACKET, 0);
	if(SockSPX < 0)
	{
		perror("SpxServer: socket");
		exit(1);
	}
	else
		printf("%s: socket(%d) complete\n", ProgName, SockSPX);
	

	/* Set socket to reuse address */
	opt = 1;
	err = setsockopt(SockSPX, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt));
        if(err < 0)
	{
                perror("SpxServer: setsockopt");
                exit(1);
        }
	else
		printf("%s: setsockopt(%d) complete\n", ProgName, err);

	/* Setup port to listen on */
	memset(&spx, 0, sizeof(spx));
	spx.sipx_family		= AF_SPX;		/* Proto family */
        spx.sipx_network 	= htonl(0xC0A80113);	/* Our network */
	spx.sipx_port 		= htons(PORT);		/* Listen port */
	spx.sipx_type 		= 5;			/* Packet type */

	/* Bind our port to the interface(s) for serving */
	err = bind(SockSPX, (struct sockaddr *)&spx, sizeof(spx));
	if(err < 0)
	{
		perror("SpxServer: bind");
		exit(1);
	}
	else
		printf("%s: bind(%d) complete\n", ProgName, err);

	len = sizeof(spx);
	err = getsockname(SockSPX, (struct sockaddr *)&spx, &len);
        if(err < 0)
	{
                perror("SpxServer: getsockname");
                exit(1);
        }
	else
		printf("%s: getsockname(%d) complete\n", ProgName, err);

	/* Set the socket up to listen for connections */
	err = listen(SockSPX, 4);
	if(err < 0)
	{
		perror("SpxServer: listen");
		exit(1);
	}
	else
		printf("%s: listen(%d) complete\n", ProgName, err);

	/* Accept will block till we have a connection */
	len = sizeof(spx);
        NewSock = accept(SockSPX, (struct sockaddr *)&spx, &len);
        if(NewSock < 0)
	{
                perror("SpxServer: accept");
                exit(1);
        }
	else
	{
		printf("%s: accept(%d) complete", ProgName, NewSock);
		printf(" from %08X:%02X%02X%02X%02X%02X%02X:%04X\n",
                	htonl(spx.sipx_network),
                	spx.sipx_node[0], spx.sipx_node[1],
                	spx.sipx_node[2], spx.sipx_node[3],
                	spx.sipx_node[4], spx.sipx_node[5],
                	htons(spx.sipx_port));
	}

	count = 1;
	while (1)
	{
		memset(msg, 0, sizeof(msg));
                result = read(NewSock, msg, sizeof(msg));
                if(result < 0)
		{
                        perror("SpxServer: socket read1: ");
			exit (result);
                }
		else
		{
			printf("%s: read %d bytes\n", ProgName, result);
			printf("%s: %s\n", ProgName, msg);
		}

		memset(msg, 0, sizeof(msg));
                sprintf(msg, "Message %d from SpxServer!", count);

                result = write(NewSock, msg, sizeof(msg));
                if(result < 0)
                {
                        perror("SpxServer: socket write1");
                        exit (result);
                }
                else
                        printf("%s: write %d bytes\n", ProgName, result);

                count++;
	}

	return;
}
